<?php

namespace WooViews\Block\Hooks;

use ToolsetCommonEs\Block\Style\Attribute\Factory as FactoryStyleAttribute;
use ToolsetCommonEs\Block\Style\Block\WithConfig;
use ToolsetCommonEs\Library\MobileDetect\MobileDetect;

/**
 * Specific hooks for Product Image block
 *
 * It needs some hooks with image size and WooCommerce filters,
 * that may depend on the Product Image shortcode that are not related to the filter parameters
 */
class ProductImage {
	/** @param array */
	private $shortcode_attributes;

	/** @param \ToolsetCommonEs\Utils\ImageResize */
	private $image_resize;

	/**
	 * Constructor
	 *
	 * @param array $shortcode_attributes List of the shortcode attributes
	 * @param ImageResize $image_resize Image resizer
	 */
	public function __construct( array $shortcode_attributes, \ToolsetCommonEs\Utils\ImageResize $image_resize ) {
		$this->shortcode_attributes = $shortcode_attributes;
		$this->image_resize = $image_resize;
	}

	/**
	 * Modifies woocommerce product image params, adding srcset and sizes
	 *
	 * @param array $image_attributes Image attributes.
	 * @param int $attachment_id Attachment ID.
	 * @param string $image_size Image size: full, thumbnail, ...
	 * @param string $main_image Original image URL
	 * @return array
	 */
	public function woocommerce_gallery_image_html_attachment_image_params( $image_attributes, $attachment_id, $image_size, $main_image ) {
		if ( isset( $this->shortcode_attributes['size'] ) &&
			$this->shortcode_attributes['size'] === 'custom' &&
			isset( $this->shortcode_attributes['width'] ) &&
			isset( $this->shortcode_attributes['height'] )
		) {
			$image_resized = $this->image_resize->get_resized_image_by_original_url(
				$image_attributes['data-src'],
				$this->shortcode_attributes['width'],
				$this->shortcode_attributes['height'],
				isset( $this->shortcode_attributes['crop'] ) && $this->shortcode_attributes['crop'] === 'true'
			);
			$image_attributes['src'] = $image_resized['url'];
			$image_attributes['srcset'] = $image_resized['url']. ' ' . $image_resized['width'] . 'w, ' .
				wp_get_attachment_image_srcset( $attachment_id );
			$image_attributes['sizes'] = "(max-width: {$image_resized['width']}px) 100vw, {$image_resized['width']}px";
		}

		return $image_attributes;
	}

	/**
	 * Modifies Product image sizes: width and height.
	 *
	 * @since 4.3.0
	 *
	 * @param array $image Image data.
	 * @param int $attachment_id Image attachment ID.
	 * @param string $size Image size.
	 * @param bool $icon Whether the image should be treated as an icon.
	 */
	public function wp_get_attachment_image_src( $image, $attachment_id, $size, $icon ) {
		if (
			isset( $this->shortcode_attributes['size'] ) &&
			$this->shortcode_attributes['size'] === 'custom' &&
			isset( $this->shortcode_attributes['width'] ) &&
			isset( $this->shortcode_attributes['height'] )
		) {
			$image[1] = $this->shortcode_attributes['width'];
			$image[2] = $this->shortcode_attributes['height'];
		}
		return $image;
	}
}
