<?php

namespace WooViews\Settings;

use WooViews\Settings\Persistence\Models\Global_Settings;
use WooViews\Settings\Sections\Provider;

/**
 * Class Frontend
 *
 * @package WooViews\Settings
 */
class Frontend {

	/**
	 * @var string
	 */
	private $toolset_section_slug = 'woo-blocks';

	/**
	 * @var Provider
	 */
	private $sections_provider;

	/**
	 * @var Global_Settings
	 */
	private $model;


	/**
	 * Frontend constructor.
	 *
	 * @param Provider $sections_provider
	 * @param Global_Settings $model
	 */
	public function __construct( Provider $sections_provider, Global_Settings $model ) {
		$this->sections_provider = $sections_provider;
		$this->model = $model;
	}


	/**
	 * Initialization hooks.
	 */
	public function init() {
		add_action( 'load-toolset_page_toolset-settings', array( $this, 'build' ) );
		add_action( 'wp_ajax_woo_blocks_store_settings', array( $this, 'store_settings' ) );
		add_action( 'wp_ajax_woo_blocks_switch_templates', array( $this, 'handle_switch_templates' ) );
	}


	/**
	 * Build frontend.
	 */
	public function build() {
		add_filter( 'toolset_filter_toolset_register_settings_section', array(
			$this,
			'register_woo_blocks_section',
		), 60 );

		$this->register_sections();
	}


	/**
	 * Registers main settings section.
	 *
	 * @param $sections
	 *
	 * @return mixed
	 */
	public function register_woo_blocks_section( $sections ) {
		$sections[ $this->toolset_section_slug ] = array(
			'slug' => $this->toolset_section_slug,
			'title' => __( 'WooCommerce Blocks', 'woocommerce-views' ),
		);

		return $sections;
	}


	/**
	 * Registers sub-settings sections.
	 */
	private function register_sections() {
		foreach ( $this->sections_provider->get() as $section ) {
			if ( $section->is_enabled() ) {
				add_filter( sprintf( 'toolset_filter_toolset_register_settings_%s_section', $this->toolset_section_slug ), array(
					$section,
					'register',
				), 10, 2 );
			}
		}
	}


	/**
	 * Handles the AJAX requests for saving settings.
	 */
	public function store_settings() {
		$defaults = [ 'wcb_frontend_sorting_enabled' => false, 'wcb_use_woocommerce_styling' => true ];

		if ( isset( $_POST['wpNonce'] ) ) {
			if ( wp_verify_nonce( sanitize_text_field( $_POST['wpNonce'] ), 'woo_blocks_store_settings' ) ) {
				$options = array_merge( $defaults, $_POST );
				$this->model->set_use_woocommerce_styling( $options['wcb_use_woocommerce_styling'] );
				$this->model->set_frontend_sorting_enabled( $options['wcb_frontend_sorting_enabled'] );

				wp_send_json_success( [ 'message' => __( 'Settings saved.', 'woocommerce-views' ) ] );
			}
		}
		wp_send_json_error( [ 'message' => __( 'Couldn\'t save the settings.', 'woocommerce-views' ) ] );
	}


	/**
	 * Handles the AJAX requests for switching to WCB templates.
	 */
	public function handle_switch_templates() {
		if ( isset( $_POST['wpNonce'] ) ) {
			if ( wp_verify_nonce( sanitize_text_field( $_POST['wpNonce'] ), 'woo_blocks_switch_templates' ) ) {
				$this->model->clean_template_path_options();
				wp_send_json_success( [ 'message' => __( 'Changes saved.', 'woocommerce-views' ) ] );
			}
		}

		wp_send_json_error( [ 'message' => __( 'Couldn\'t save the changes.', 'woocommerce-views' ) ] );
	}
}
